@@ -1,16 +1,11 @@
-# Copyright (C) 2002, Thomas Hamelryck (thamelry@vub.ac.be)
+# Copyright (C) 2002, Thomas Hamelryck (thamelry@binf.ku.dk)
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.  
 
+"""Chain class, used in Structure objects."""
 
-# Python stuff
-from types import IntType
-
-# My Stuff
-from Entity import Entity
-
-__doc__="Chain class, used in Structure objects."
+from Bio.PDB.Entity import Entity
 
 
 class Chain(Entity):
@@ -44,7 +39,7 @@ class Chain(Entity):
         """
         A residue id is normally a tuple (hetero flag, sequence identifier, 
         insertion code). Since for most residues the hetero flag and the 
-        insertion code are blanc (i.e. " "), you can just use the sequence 
+        insertion code are blank (i.e. " "), you can just use the sequence 
         identifier to index a residue in a chain. The _translate_id method
         translates the sequence identifier to the (" ", sequence identifier,
         " ") tuple. 
@@ -52,7 +47,7 @@ class Chain(Entity):
         Arguments:
         o id - int, residue resseq 
         """
-        if type(id)==IntType:
+        if isinstance(id, int):
             id=(' ', id, ' ')
         return id
             
@@ -71,6 +66,15 @@ class Chain(Entity):
         id=self._translate_id(id)
         return Entity.__getitem__(self, id)
 
+    def __contains__(self, id):
+        """True if a residue with given id is present in this chain.
+
+        Arguments:
+        o id - (string, int, string) or int
+        """
+        id=self._translate_id(id)
+        return Entity.__contains__(self, id)
+
     def __delitem__(self, id):
         """
         Arguments:
@@ -116,6 +120,10 @@ class Chain(Entity):
 
     # Public
 
+    def get_residues(self):
+        for r in self:
+            yield r
+
     def get_atoms(self):
         for r in self:
             for a in r:
