@@ -7,11 +7,10 @@
 
 import os.path
 import warnings
-import Bio.PDB.mmCIF.MMCIFlex
-from UserDict import UserDict
+import Bio.PDB.mmCIF.MMCIFlex as MMCIFlex
 
 
-class MMCIF2Dict(UserDict):
+class MMCIF2Dict(dict):
     # The token identifiers
     NAME=1
     LOOP=2
@@ -22,17 +21,21 @@ class MMCIF2Dict(UserDict):
     SIMPLE=7
 
     def __init__(self, filename):
-        # this dict will contain the name/data pairs 
-        self.data={}
-        # entry for garbage
-        self.data[None]=[]
         if not os.path.isfile(filename):
             raise IOError("File not found.")
-        Bio.PDB.mmCIF.MMCIFlex.open_file(filename)
-        self._make_mmcif_dict()
-        Bio.PDB.mmCIF.MMCIFlex.close_file()
+        MMCIFlex.open_file(filename)
+        dict.__init__(self, **self._make_mmcif_dict())
+        MMCIFlex.close_file()
 
     def _make_mmcif_dict(self): 
+        """
+        Loop through PLY token (type, value) pairs, return a dict.
+
+        """
+        # this dict will contain the name/data pairs 
+        mmcif_dict = {}
+        # entry for garbage
+        mmcif_dict[None] = []
         # local copies
         NAME=self.NAME
         LOOP=self.LOOP
@@ -41,7 +44,7 @@ class MMCIF2Dict(UserDict):
         DOUBLEQUOTED=self.DOUBLEQUOTED
         QUOTED=self.QUOTED
         SIMPLE=self.SIMPLE
-        get_token=Bio.PDB.mmCIF.MMCIFlex.get_token
+        get_token=MMCIFlex.get_token
         # are we looping?
         loop_flag=0
         # list of names in loop
@@ -51,7 +54,6 @@ class MMCIF2Dict(UserDict):
         # get first token/value pair
         token, value=get_token()
         # print token, value
-        mmcif_dict=self.data
         # loop until EOF (token==0)
         while token:
             if token==NAME:
@@ -114,9 +116,7 @@ class MMCIF2Dict(UserDict):
             if token==None:
                 token, value=get_token()
                 # print token, value
-
-    def __getitem__(self, key):
-        return self.data[key]
+        return mmcif_dict
 
 
 if __name__=="__main__":
@@ -128,7 +128,7 @@ if __name__=="__main__":
 
     filename=sys.argv[1]    
 
-    mmcif_dict=MMCIF2Dict(filename)
+    mmcif_dict = MMCIF2Dict(filename)
 
     entry = ""
     print "Now type a key ('q' to end, 'k' for a list of all keys):"
