@@ -314,7 +314,7 @@ class TreeMixin(object):
         >>> matches.next()
         Taxonomy(code='OCTVU', scientific_name='Octopus vulgaris')
 
-        """ 
+        """
         if terminal is not None:
             kwargs['terminal'] = terminal
         is_matching_elem = _combine_matchers(target, kwargs, False)
@@ -390,7 +390,7 @@ class TreeMixin(object):
     def common_ancestor(self, targets, *more_targets):
         """Most recent common ancestor (clade) of all the given targets.
 
-        Edge cases: 
+        Edge cases:
         - If no target is given, returns self.root
         - If 1 target is given, returns the target
         - If any target is not found in this tree, raises a ValueError
@@ -440,7 +440,7 @@ class TreeMixin(object):
             for child in node.clades:
                 new_depth = curr_depth + depth_of(child)
                 update_depths(child, new_depth)
-        update_depths(self.root, 0)
+        update_depths(self.root, self.root.branch_length or 0)
         return depths
 
     def distance(self, target1, target2=None):
@@ -456,7 +456,7 @@ class TreeMixin(object):
 
     def is_bifurcating(self):
         """Return True if tree downstream of node is strictly bifurcating.
-        
+
         I.e., all nodes have either 2 or 0 children (internal or external,
         respectively). The root may have 3 descendents and still be considered
         part of a bifurcating tree, because it has no ancestor.
@@ -509,7 +509,7 @@ class TreeMixin(object):
         """True if target is a descendent of this tree.
 
         Not required to be a direct descendent.
-        
+
         To check only direct descendents of a clade, simply use list membership
         testing: ``if subclade in clade: ...``
         """
@@ -580,11 +580,14 @@ class TreeMixin(object):
         function is straightforward.
         """
         # Read the iterable into a list to protect against in-place changes
-        internals = list(self.find_clades(target, False, 'level', **kwargs))
+        matches = list(self.find_clades(target, False, 'level', **kwargs))
+        if not matches:
+            # No matching nodes to collapse
+            return
         # Skip the root node -- it can't be collapsed
-        if internals[0] == self.root:
-            internals.pop(0)
-        for clade in internals:
+        if matches[0] == self.root:
+            matches.pop(0)
+        for clade in matches:
             self.collapse(clade)
 
     def ladderize(self, reverse=False):
@@ -740,7 +743,9 @@ class Tree(TreeElement, TreeMixin):
         from Bio.Phylo.PhyloXML import Phylogeny
         return Phylogeny.from_tree(self, **kwargs)
 
-    def root_with_outgroup(self, outgroup_targets, *more_targets):
+    # XXX Compatibility: In Python 2.6+, **kwargs can be replaced with the named
+    # keyword argument outgroup_branch_length=None
+    def root_with_outgroup(self, outgroup_targets, *more_targets, **kwargs):
         """Reroot this tree with the outgroup clade containing outgroup_targets.
 
         Operates in-place.
@@ -754,6 +759,14 @@ class Tree(TreeElement, TreeMixin):
           trifurcating root, keeping branches the same
         - If the original root was bifurcating, drop it from the tree,
           preserving total branch lengths
+
+        :param outgroup_branch_length: length of the branch leading to the
+            outgroup after rerooting. If not specified (None), then:
+
+            - If the outgroup is an internal node (not a single terminal taxon),
+              then use that node as the new root.
+            - Otherwise, create a new root node as the parent of the outgroup.
+
         """
         # This raises a ValueError if any target is not in this tree
         # Otherwise, common_ancestor guarantees outgroup is in this tree
@@ -763,20 +776,31 @@ class Tree(TreeElement, TreeMixin):
             # Outgroup is the current root -- no change
             return
 
-        prev_blen = outgroup.branch_length
-        if outgroup.is_terminal():
+        prev_blen = outgroup.branch_length or 0.0
+        # Hideous kludge because Py2.x doesn't allow keyword args after *args
+        outgroup_branch_length = kwargs.get('outgroup_branch_length')
+        if outgroup_branch_length is not None:
+            assert 0 <= outgroup_branch_length <= prev_blen, \
+                    "outgroup_branch_length must be between 0 and the " \
+                    "original length of the branch leading to the outgroup."
+
+        if outgroup.is_terminal() or outgroup_branch_length is not None:
             # Create a new root with a 0-length branch to the outgroup
-            outgroup.branch_length = 0.0
+            outgroup.branch_length = outgroup_branch_length or 0.0
             new_root = self.root.__class__(
                     branch_length=self.root.branch_length, clades=[outgroup])
             # The first branch reversal (see the upcoming loop) is modified
             if len(outgroup_path) == 1:
-                # Trivial tree like '(A,B);
+                # No nodes between the original root and outgroup to rearrange.
+                # Most of the code below will be skipped, but we still need
+                # 'new_parent' pointing at the new root.
                 new_parent = new_root
             else:
                 parent = outgroup_path.pop(-2)
+                # First iteration of reversing the path to the outgroup
                 parent.clades.pop(parent.clades.index(outgroup))
-                prev_blen, parent.branch_length = parent.branch_length, prev_blen
+                (prev_blen, parent.branch_length) = (parent.branch_length,
+                        prev_blen - outgroup.branch_length)
                 new_root.clades.insert(0, parent)
                 new_parent = parent
         else:
@@ -819,13 +843,48 @@ class Tree(TreeElement, TreeMixin):
         self.rooted = True
         return
 
+    def root_at_midpoint(self):
+        """Root the tree at the midpoint of the two most distant taxa.
+
+        This operates in-place, leaving a bifurcating root. The topology of the
+        tree is otherwise retained, though no guarantees are made about the
+        stability of clade/node/taxon ordering.
+        """
+        # Identify the largest pairwise distance
+        max_distance = 0.0
+        tips = self.get_terminals()
+        for tip in tips:
+            self.root_with_outgroup(tip)
+            new_max = max(self.depths().iteritems(), key=lambda nd: nd[1])
+            if new_max[1] > max_distance:
+                tip1 = tip
+                tip2 = new_max[0]
+                max_distance = new_max[1]
+        self.root_with_outgroup(tip1)
+        # Depth to go from the ingroup tip toward the outgroup tip
+        root_remainder = 0.5 * (max_distance - (self.root.branch_length or 0))
+        assert root_remainder >= 0
+        # Identify the midpoint and reroot there.
+        # Trace the path to the outgroup tip until all of the root depth has
+        # been traveled/accounted for.
+        for node in self.get_path(tip2):
+            root_remainder -= node.branch_length
+            if root_remainder < 0:
+                outgroup_node = node
+                outgroup_branch_length = -root_remainder
+                break
+        else:
+            raise ValueError("Somehow, failed to find the midpoint!")
+        self.root_with_outgroup(outgroup_node,
+                                outgroup_branch_length=outgroup_branch_length)
+
     # Method assumed by TreeMixin
 
     def is_terminal(self):
         """True if the root of this tree is terminal."""
         return (not self.root.clades)
 
-    # Convention from SeqRecord and Alignment classes  
+    # Convention from SeqRecord and Alignment classes
 
     def __format__(self, format_spec):
         """Serialize the tree as a string in the specified file format.
@@ -892,13 +951,21 @@ class Clade(TreeElement, TreeMixin):
             The clade's name (a label).
         clades : list
             Sub-trees rooted directly under this tree's root.
+        confidence : number
+            Support.
+        color : BranchColor
+            The display color of the branch and descendents.
+        width : number
+            The display width of the branch and descendents.
     """
     def __init__(self, branch_length=None, name=None, clades=None,
-            confidence=None):
+            confidence=None, color=None, width=None):
         self.branch_length = branch_length
         self.name = name
         self.clades = clades or []
         self.confidence = confidence
+        self.color = color
+        self.width = width
 
     @property
     def root(self):
@@ -941,3 +1008,145 @@ class Clade(TreeElement, TreeMixin):
         if self.name:
             return _sugar.trim_str(self.name, maxlen=40)
         return self.__class__.__name__
+
+    # Syntax sugar for setting the branch color
+    def _get_color(self):
+        return self._color
+
+    def _set_color(self, arg):
+        if arg is None or isinstance(arg, BranchColor):
+            self._color = arg
+        elif isinstance(arg, basestring):
+            if arg in BranchColor.color_names:
+                # Known color name
+                self._color = BranchColor.from_name(arg)
+            elif arg.startswith('#') and len(arg) == 7:
+                # HTML-style hex string
+                self._color = BranchColor.from_hex(arg)
+            else:
+                raise ValueError("invalid color string %s" % arg)
+        elif hasattr(arg, '__iter__') and len(arg) == 3:
+            # RGB triplet
+            self._color = BranchColor(*arg)
+        else:
+            raise ValueError("invalid color value %s" % arg)
+
+    color = property(_get_color, _set_color, doc="Branch color.")
+
+
+class BranchColor(object):
+    """Indicates the color of a clade when rendered graphically.
+
+    The color should be interpreted by client code (e.g. visualization
+    programs) as applying to the whole clade, unless overwritten by the
+    color(s) of sub-clades.
+
+    Color values must be integers from 0 to 255.
+    """
+
+    color_names = {
+            'red':      (255,   0,   0),
+            'r':        (255,   0,   0),
+            'yellow':   (255, 255,   0),
+            'y':        (255, 255,   0),
+            'green':    (  0, 128,   0),
+            'g':        (  0, 128,   0),
+            'cyan':     (  0, 255, 255),
+            'c':        (  0, 255, 255),
+            'blue':     (  0,   0, 255),
+            'b':        (  0,   0, 255),
+            'magenta':  (255,   0, 255),
+            'm':        (255,   0, 255),
+            'black':    (  0,   0,   0),
+            'k':        (  0,   0,   0),
+            'white':    (255, 255, 255),
+            'w':        (255, 255, 255),
+            # Names standardized in HTML/CSS spec
+            # http://w3schools.com/html/html_colornames.asp
+            'maroon':   (128,   0,   0),
+            'olive':    (128, 128,   0),
+            'lime':     (  0, 255,   0),
+            'aqua':     (  0, 255, 255),
+            'teal':     (  0, 128, 128),
+            'navy':     (  0,   0, 128),
+            'fuchsia':  (255,   0, 255),
+            'purple':   (128,   0, 128),
+            'silver':   (192, 192, 192),
+            'gray':     (128, 128, 128),
+            # More definitions from matplotlib/gcolor2
+            'grey':     (128, 128, 128),
+            'pink':     (255, 192, 203),
+            'salmon':   (250, 128, 114),
+            'orange':   (255, 165,   0),
+            'gold':     (255, 215,   0),
+            'tan':      (210, 180, 140),
+            'brown':    (165,  42,  42),
+            }
+
+    def __init__(self, red, green, blue):
+        for color in (red, green, blue):
+            assert (isinstance(color, int) and
+                    0 <= color <= 255
+                    ), "Color values must be integers between 0 and 255."
+        self.red = red
+        self.green = green
+        self.blue = blue
+
+    @classmethod
+    def from_hex(cls, hexstr):
+        """Construct a BranchColor object from a hexadecimal string.
+
+        The string format is the same style used in HTML and CSS, such as
+        '#FF8000' for an RGB value of (255, 128, 0).
+        """
+        assert (isinstance(hexstr, basestring) and
+                hexstr.startswith('#') and
+                len(hexstr) == 7
+                ), "need a 24-bit hexadecimal string, e.g. #000000"
+        def unpack(cc):
+            return int('0x'+cc, base=16)
+        RGB = hexstr[1:3], hexstr[3:5], hexstr[5:]
+        return cls(*map(unpack, RGB))
+
+    @classmethod
+    def from_name(cls, colorname):
+        """Construct a BranchColor object by the color's name."""
+        return cls(*cls.color_names[colorname])
+
+    def to_hex(self):
+        """Return a 24-bit hexadecimal RGB representation of this color.
+
+        The returned string is suitable for use in HTML/CSS, as a color
+        parameter in matplotlib, and perhaps other situations.
+
+        Example:
+
+            >>> bc = BranchColor(12, 200, 100)
+            >>> bc.to_hex()
+            '#0cc864'
+        """
+        return '#' + hex(
+                self.red * (16**4)
+                + self.green * (16**2)
+                + self.blue)[2:].zfill(6)
+
+    def to_rgb(self):
+        """Return a tuple of RGB values (0 to 255) representing this color.
+
+        Example:
+
+            >>> bc = BranchColor(255, 165, 0)
+            >>> bc.to_rgb()
+            (255, 165, 0)
+        """
+        return (self.red, self.green, self.blue)
+
+    def __repr__(self):
+        """Preserve the standard RGB order when representing this object."""
+        return (u'%s(red=%d, green=%d, blue=%d)'
+                % (self.__class__.__name__, self.red, self.green, self.blue))
+
+    def __str__(self):
+        """Show the color's RGB values."""
+        return "(%d, %d, %d)" % (self.red, self.green, self.blue)
+
