@@ -883,11 +883,14 @@ def FastqGeneralIterator(handle):
     #Skip any text before the first record (e.g. blank lines, comments?)
     while True:
         line = handle_readline()
-        if line == "" : return #Premature end of file, or just empty?
+        if not line:
+            return #Premature end of file, or just empty?
         if line[0] == "@":
             break
+        if isinstance(line[0], int):
+            raise ValueError("Is this handle in binary mode not text mode?")
 
-    while True:
+    while line:
         if line[0] != "@":
             raise ValueError("Records in Fastq files should start with '@' character")
         title_line = line[1:].rstrip()
@@ -938,8 +941,8 @@ def FastqGeneralIterator(handle):
 
         #Return the record and then continue...
         yield (title_line, seq_string, quality_string)
-        if not line : return #StopIteration at end of file
-    assert False, "Should not reach this line"
+    raise StopIteration
+
         
 #This is a generator function!
 def FastqPhredIterator(handle, alphabet = single_letter_alphabet, title2ids = None):
