@@ -31,12 +31,21 @@ from Bio import SeqUtils
 
 # Bio.Graphics.GenomeDiagram
 from Bio.Graphics.GenomeDiagram import FeatureSet, GraphSet, Track, Diagram
+from Bio.Graphics.GenomeDiagram import CrossLink
 #from Bio.Graphics.GenomeDiagram.Utilities import *
 
 #Currently private, but we test them here:
 from Bio.Graphics.GenomeDiagram._Graph import GraphData
 from Bio.Graphics.GenomeDiagram._Colors import ColorTranslator
 
+def fill_and_border(base_color, alpha=0.5):
+    try:
+        c = base_color.clone()
+        c.alpha = alpha
+        return c, base_color
+    except AttributeError:
+        #Old ReportLab, no transparency and/or no clone
+        return base_color, base_color
 
 ###############################################################################
 # Utility functions for graph plotting, originally in GenomeDiagram.Utilities #
@@ -228,13 +237,11 @@ class GraphTest(unittest.TestCase):
                  fragments=1,
                  start=0, end=points)
         gdd.write(os.path.join('Graphics', "line_graph.pdf"), "pdf")
-        #Circular diagram - move tracks to make an empty space in the middle
-        for track_number in gdd.tracks:
-            gdd.move_track(track_number,track_number+1)
+        #Circular diagram
         gdd.draw(tracklines=False,
                  pagesize=(15*cm,15*cm),
                  circular=True, #Data designed to be periodic
-                 start=0, end=points)
+                 start=0, end=points, circle_core=0.5)
         gdd.write(os.path.join('Graphics', "line_graph_c.pdf"), "pdf")
         
     def test_slicing(self):
@@ -272,12 +279,11 @@ class LabelTest(unittest.TestCase):
         #For the tutorial this might be useful:
         #self.gdd.write(os.path.join('Graphics', name+".png"), "png")
         if circular:
-            #Circular diagram - move tracks to make an empty space in the middle
-            for track_number in self.gdd.tracks:
-                self.gdd.move_track(track_number,track_number+1)
+            #Circular diagram
             self.gdd.draw(tracklines=False,
                           pagesize=(15*cm,15*cm),
                           fragments=1,
+                          circle_core=0.5,
                           start=0, end=400)
             self.gdd.write(os.path.join('Graphics', name+"_c.pdf"), "pdf")
     
@@ -348,12 +354,11 @@ class SigilsTest(unittest.TestCase):
         #For the tutorial this might be useful:
         #self.gdd.write(os.path.join('Graphics', name+".png"), "png")
         if circular:
-            #Circular diagram - move tracks to make an empty space in the middle
-            for track_number in self.gdd.tracks:
-                self.gdd.move_track(track_number,track_number+1)
+            #Circular diagram
             self.gdd.draw(tracklines=False,
                           pagesize=(15*cm,15*cm),
                           fragments=1,
+                          circle_core=0.5,
                           start=0, end=400)
             self.gdd.write(os.path.join('Graphics', name+"_c.pdf"), "pdf")
 
@@ -653,10 +658,7 @@ class DiagramTest(unittest.TestCase):
         output_filename = os.path.join('Graphics', 'GD_by_meth_linear.pdf')
         gdd.write(output_filename, 'PDF')
 
-        #Change the order and leave an empty space in the center:
-        gdd.move_track(1,3)
-
-        gdd.draw(format='circular', tracklines=False,
+        gdd.draw(format='circular', tracklines=False, circle_core=0.8,
                  pagesize=(20*cm,20*cm), circular=True)
         output_filename = os.path.join('Graphics', 'GD_by_meth_circular.pdf')
         gdd.write(output_filename, 'PDF')
@@ -666,23 +668,106 @@ class DiagramTest(unittest.TestCase):
         genbank_entry = self.record
         gdd = Diagram('Test Diagram')
 
+        gdt1 = Track('CDS features', greytrack=True,
+                     scale_largetick_interval=1e4,
+                     scale_smalltick_interval=1e3,
+                     greytrack_labels=10,
+                     greytrack_font_color="red",
+                     scale_format = "SInt")
+        gdt2 = Track('gene features', greytrack=1,
+                   scale_largetick_interval=1e4)
+
         #First add some feature sets:
+        gdfsA = FeatureSet(name='CDS backgrounds')
+        gdfsB = FeatureSet(name='gene background')
+
+
         gdfs1 = FeatureSet(name='CDS features')
         gdfs2 = FeatureSet(name='gene features')
         gdfs3 = FeatureSet(name='misc_features')
         gdfs4 = FeatureSet(name='repeat regions')
 
+        prev_gene = None
+        cds_count = 0
+        for feature in genbank_entry.features:
+            if feature.type == 'CDS':
+                cds_count += 1
+                if prev_gene:
+                    #Assuming it goes with this CDS!
+                    if cds_count % 2 == 0:
+                        dark, light = colors.peru, colors.tan
+                    else:
+                        dark, light = colors.burlywood, colors.bisque
+                    #Background for CDS,
+                    a = gdfsA.add_feature(SeqFeature(FeatureLocation(feature.location.start, feature.location.end, strand=0)),
+                                         color=dark)
+                    #Background for gene,
+                    b = gdfsB.add_feature(SeqFeature(FeatureLocation(prev_gene.location.start, prev_gene.location.end, strand=0)),
+                                          color=dark)
+                    #Cross link,
+                    gdd.cross_track_links.append(CrossLink(a, b, light, dark))
+                    prev_gene = None
+            if feature.type == 'gene':
+                prev_gene = feature
+
+        #Some cross links on the same linear diagram fragment,
+        f, c = fill_and_border(colors.red)
+        a = gdfsA.add_feature(SeqFeature(FeatureLocation(2220,2230)), color=f, border=c)
+        b = gdfsB.add_feature(SeqFeature(FeatureLocation(2200,2210)), color=f, border=c)
+        gdd.cross_track_links.append(CrossLink(a, b, f, c))
+
+        f, c = fill_and_border(colors.blue)
+        a = gdfsA.add_feature(SeqFeature(FeatureLocation(2150,2200)), color=f, border=c)
+        b = gdfsB.add_feature(SeqFeature(FeatureLocation(2220,2290)), color=f, border=c)
+        gdd.cross_track_links.append(CrossLink(a, b, f, c, flip=True))
+
+        f, c = fill_and_border(colors.green)
+        a = gdfsA.add_feature(SeqFeature(FeatureLocation(2250,2560)), color=f, border=c)
+        b = gdfsB.add_feature(SeqFeature(FeatureLocation(2300,2860)), color=f, border=c)
+        gdd.cross_track_links.append(CrossLink(a, b, f, c))
+
+        #Some cross links where both parts are saddling the linear diagram fragment boundary,
+        f, c = fill_and_border(colors.red)
+        a = gdfsA.add_feature(SeqFeature(FeatureLocation(3155,3250)), color=f, border=c)
+        b = gdfsB.add_feature(SeqFeature(FeatureLocation(3130,3300)), color=f, border=c)
+        gdd.cross_track_links.append(CrossLink(a, b, f, c))
+        #Nestled within that (drawn on top),
+        f, c = fill_and_border(colors.blue)
+        a = gdfsA.add_feature(SeqFeature(FeatureLocation(3160,3275)), color=f, border=c)
+        b = gdfsB.add_feature(SeqFeature(FeatureLocation(3180,3225)), color=f, border=c)
+        gdd.cross_track_links.append(CrossLink(a, b, f, c, flip=True))
+
+        #Some cross links where two features are on either side of the linear diagram fragment boundary,
+        f, c = fill_and_border(colors.green)
+        a = gdfsA.add_feature(SeqFeature(FeatureLocation(6450,6550)), color=f, border=c)
+        b = gdfsB.add_feature(SeqFeature(FeatureLocation(6265,6365)), color=f, border=c)
+        gdd.cross_track_links.append(CrossLink(a, b, color=f, border=c))
+        f, c = fill_and_border(colors.gold)
+        a = gdfsA.add_feature(SeqFeature(FeatureLocation(6265,6365)), color=f, border=c)
+        b = gdfsB.add_feature(SeqFeature(FeatureLocation(6450,6550)), color=f, border=c)
+        gdd.cross_track_links.append(CrossLink(a, b, color=f, border=c))
+        f, c = fill_and_border(colors.red)
+        a = gdfsA.add_feature(SeqFeature(FeatureLocation(6275,6375)), color=f, border=c)
+        b = gdfsB.add_feature(SeqFeature(FeatureLocation(6430,6530)), color=f, border=c)
+        gdd.cross_track_links.append(CrossLink(a, b, color=f, border=c, flip=True))
+        f, c = fill_and_border(colors.blue)
+        a = gdfsA.add_feature(SeqFeature(FeatureLocation(6430,6530)), color=f, border=c)
+        b = gdfsB.add_feature(SeqFeature(FeatureLocation(6275,6375)), color=f, border=c)
+        gdd.cross_track_links.append(CrossLink(a, b, color=f, border=c, flip=True))
+
+
         cds_count = 0
         for feature in genbank_entry.features:
             if feature.type == 'CDS':
                 cds_count += 1
                 if cds_count % 2 == 0:
-                    gdfs1.add_feature(feature, color=colors.pink)
+                    gdfs1.add_feature(feature, color=colors.pink, sigil="ARROW")
                 else:
-                    gdfs1.add_feature(feature, color=colors.red)
+                    gdfs1.add_feature(feature, color=colors.red, sigil="ARROW")
 
             if feature.type == 'gene':
-                gdfs2.add_feature(feature)
+                #Note we set the colour of ALL the genes later on as a test,
+                gdfs2.add_feature(feature, sigil="ARROW")
 
             if feature.type == 'misc_feature':
                 gdfs3.add_feature(feature, color=colors.orange)
@@ -690,6 +775,7 @@ class DiagramTest(unittest.TestCase):
             if feature.type == 'repeat_region':
                 gdfs4.add_feature(feature, color=colors.purple)
 
+        #gdd.cross_track_links = gdd.cross_track_links[:1]
 
         gdfs1.set_all_features('label', 1)
         gdfs2.set_all_features('label', 1)
@@ -702,16 +788,10 @@ class DiagramTest(unittest.TestCase):
         #gdfs1.set_all_features('color', colors.red)
         gdfs2.set_all_features('color', colors.blue)
 
-        gdt1 = Track('CDS features', greytrack=True,
-                     scale_largetick_interval=1e4,
-                     scale_smalltick_interval=1e3,
-                     greytrack_labels=10,
-                     greytrack_font_color="red",
-                     scale_format = "SInt")
+        gdt1.add_set(gdfsA) #Before CDS so under them!
         gdt1.add_set(gdfs1)
 
-        gdt2 = Track('gene features', greytrack=1,
-                   scale_largetick_interval=1e4)
+        gdt2.add_set(gdfsB) #Before genes so under them!
         gdt2.add_set(gdfs2)
                 
         gdt3 = Track('misc features and repeats', greytrack=1,
@@ -764,7 +844,7 @@ class DiagramTest(unittest.TestCase):
         gdt6.add_set(gdgs3)
 
         #Add the tracks (from both features and graphs)
-        #Leave some white space in the middle
+        #Leave some white space in the middle/bottom
         gdd.add_track(gdt4, 3) # GC skew
         gdd.add_track(gdt5, 4) # GC and AT content
         gdd.add_track(gdt1, 5) # CDS features
@@ -772,17 +852,30 @@ class DiagramTest(unittest.TestCase):
         gdd.add_track(gdt3, 7) # Misc features and repeat feature
         gdd.add_track(gdt6, 8) # Feature depth
 
-        #Finally draw it in both formats,
+        #Finally draw it in both formats, and full view and partial
         gdd.draw(format='circular', orientation='landscape',
-             tracklines=0, pagesize='A0', circular=True)
+             tracklines=0, pagesize='A0')
         output_filename = os.path.join('Graphics', 'GD_by_obj_circular.pdf')
         gdd.write(output_filename, 'PDF')
 
+        gdd.circular=False
+        gdd.draw(format='circular', orientation='landscape',
+             tracklines=0, pagesize='A0', start=3000, end=6300)
+        output_filename = os.path.join('Graphics', 'GD_by_obj_frag_circular.pdf')
+        gdd.write(output_filename, 'PDF')
+
         gdd.draw(format='linear', orientation='landscape',
              tracklines=0, pagesize='A0', fragments=3)
         output_filename = os.path.join('Graphics', 'GD_by_obj_linear.pdf')
         gdd.write(output_filename, 'PDF')
 
+        gdd.set_all_tracks("greytrack_labels", 2)
+        gdd.draw(format='linear', orientation='landscape',
+             tracklines=0, pagesize=(30*cm,10*cm), fragments=1,
+             start=3000, end=6300)
+        output_filename = os.path.join('Graphics', 'GD_by_obj_frag_linear.pdf')
+        gdd.write(output_filename, 'PDF')
+
 if __name__ == "__main__":
     runner = unittest.TextTestRunner(verbosity = 2)
     unittest.main(testRunner=runner)
