@@ -37,17 +37,17 @@ class ModTest(unittest.TestCase):
     bad_ctl_file3 = os.path.join(ctl_dir, "bad3.ctl")
     ctl_file = os.path.join(ctl_dir, "codeml", "codeml.ctl")
        
-    def __del__(self):
+    def tearDown(self):
         """Just in case CODEML creates some junk files, do a clean-up."""
-        del_files = [self.out_file, "2NG.dN", 
-            "2NG.dS", "2NG.t", "codeml.ctl", "lnf", "rst", "rst1", 
-            "rub"]
+        del_files = [self.out_file, "2NG.dN", "2NG.dS", "2NG.t", "codeml.ctl", 
+                "lnf", "rst", "rst1", "rub"]
         for filename in del_files:
             if os.path.exists(filename):
                 os.remove(filename)
         if os.path.exists(self.working_dir):
             for filename in os.listdir(self.working_dir):
-                os.remove(filename)
+                filepath = os.path.join(self.working_dir, filename)
+                os.remove(filepath)
             os.rmdir(self.working_dir)
     
     def setUp(self):
@@ -261,6 +261,37 @@ class ModTest(unittest.TestCase):
                 if "site classes" in params:
                     self.assertEqual(len(params["site classes"]),
                                  SITECLASSES[model_num], version_msg)
+
+    def testParseNSsite3(self):
+        res_dir = os.path.join(self.results_dir, "codeml", "NSsite3")
+        for results_file in os.listdir(res_dir):
+            version = results_file.split('-')[1].split('.')[0]
+            version_msg = "Improper parsing for version %s" \
+                        % version.replace('_', '.')                    
+            results_path = os.path.join(res_dir, results_file)
+            results = codeml.read(results_path)
+            # There should be 5 top-level items: 'codon model', 'model', 
+            # 'version', 'NSsites' & site-class model, the last of which
+            # is only there when only one NSsites class is used
+            self.assertEqual(len(results), 5, version_msg)
+            self.assertTrue('site-class model' in results, version_msg)
+            self.assertEqual(results['site-class model'], 'discrete', 
+                    version_msg)
+            self.assertTrue("NSsites" in results, version_msg)
+            # There should be 1 NSsites classe: 3
+            self.assertEqual(len(results["NSsites"]), 1, version_msg)
+            # Each site class model should have 5 sub-items: 'lnL', 'tree', 
+            # 'description', 'parameters', & 'tree length'. It should
+            # have the correct number of parameters also.
+            model = results["NSsites"][3]
+            self.assertEqual(len(model), 5, version_msg)
+            self.assertTrue("parameters" in model, version_msg)
+            params = model["parameters"]
+            self.assertEqual(len(params), SITECLASS_PARAMS[3],
+                version)
+            self.assertTrue("site classes" in params, version_msg)
+            site_classes = params["site classes"]
+            self.assertEqual(len(site_classes), 4, version_msg)
         
     def testParseBranchSiteA(self):
         res_dir = os.path.join(self.results_dir, "codeml", "branchsiteA")
@@ -331,8 +362,8 @@ class ModTest(unittest.TestCase):
                 branches = site_class["branch types"]
                 self.assertEqual(len(branches), 2, version_msg)
 
-    def testParseNgene2Mgene012(self):
-        res_dir = os.path.join(self.results_dir, "codeml", "ngene2_mgene012")
+    def testParseNgene2Mgene02(self):
+        res_dir = os.path.join(self.results_dir, "codeml", "ngene2_mgene02")
         for results_file in os.listdir(res_dir):
             version = results_file.split('-')[1].split('.')[0]
             version_msg = "Improper parsing for version %s" \
@@ -354,6 +385,24 @@ class ModTest(unittest.TestCase):
             rates = params["rates"]
             self.assertEqual(len(rates), 2, version_msg)        
 
+    def testParseNgene2Mgene1(self):
+        res_dir = os.path.join(self.results_dir, "codeml", "ngene2_mgene1")
+        for results_file in os.listdir(res_dir):
+            version = results_file.split('-')[1].split('.')[0]
+            version_msg = "Improper parsing for version %s" \
+                        % version.replace('_', '.')
+            results_path = os.path.join(res_dir, results_file)
+            results = codeml.read(results_path)
+            self.assertEqual(len(results), 4, version_msg)
+            self.assertTrue("genes" in results, version_msg)
+            genes = results["genes"]
+            self.assertEqual(len(genes), 2, version_msg)
+            model = genes[0]
+            self.assertEqual(len(model), 5, version_msg)
+            self.assertTrue("parameters" in model, version_msg)
+            params = model["parameters"]
+            self.assertEqual(len(params), SITECLASS_PARAMS[0], version_msg)
+
     def testParseNgene2Mgene34(self):
         res_dir = os.path.join(self.results_dir, "codeml", "ngene2_mgene34")
         for results_file in os.listdir(res_dir):
