@@ -94,6 +94,18 @@ class TreeTests(unittest.TestCase):
         tree.root_with_outgroup('19_NEMVE', '20_NEMVE')
         self.assertEqual(orig_num_tips, len(tree.get_terminals()))
         self.assertAlmostEqual(orig_tree_len, tree.total_branch_length())
+        # Now, at an external node
+        tree.root_with_outgroup('1_BRAFL')
+        self.assertEqual(orig_num_tips, len(tree.get_terminals()))
+        self.assertAlmostEqual(orig_tree_len, tree.total_branch_length())
+        # Specifying outgroup branch length mustn't change the total tree size
+        tree.root_with_outgroup('2_BRAFL', outgroup_branch_length=0.5)
+        self.assertEqual(orig_num_tips, len(tree.get_terminals()))
+        self.assertAlmostEqual(orig_tree_len, tree.total_branch_length())
+        tree.root_with_outgroup('36_BRAFL', '37_BRAFL',
+                outgroup_branch_length=0.5)
+        self.assertEqual(orig_num_tips, len(tree.get_terminals()))
+        self.assertAlmostEqual(orig_tree_len, tree.total_branch_length())
         # On small contrived trees, testing edge cases
         for small_nwk in (
                 '(A,B,(C,D));',
@@ -108,6 +120,24 @@ class TreeTests(unittest.TestCase):
                 self.assertAlmostEqual(orig_tree_len,
                                        tree.total_branch_length())
 
+    def test_root_at_midpoint(self):
+        """Tree.root_at_midpoint: reroot at the tree's midpoint."""
+        for treefname, fmt in [(EX_APAF, 'phyloxml'),
+                               (EX_BCL2, 'phyloxml'),
+                               (EX_NEWICK, 'newick'),
+                              ]:
+            tree = Phylo.read(treefname, fmt)
+            orig_tree_len = tree.total_branch_length()
+            # Total branch length does not change
+            tree.root_at_midpoint()
+            self.assertAlmostEqual(orig_tree_len, tree.total_branch_length())
+            # Root is bifurcating
+            self.assertEqual(len(tree.root.clades), 2)
+            # Deepest tips under each child of the root are equally deep
+            deep_dist_0 = max(tree.clade[0].depths().itervalues())
+            deep_dist_1 = max(tree.clade[1].depths().itervalues())
+            self.assertAlmostEqual(deep_dist_0, deep_dist_1)
+
     # Magic method
     def test_str(self):
         """Tree.__str__: pretty-print to a string.
