@@ -1,4 +1,4 @@
-# Copyright 2009-2011 by Peter Cock.  All rights reserved.
+# Copyright 2009-2012 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -12,8 +12,10 @@ except ImportError:
     #where we don't expect this to be installed.
     sqlite3 = None
 
+import sys
 import os
 import unittest
+import gzip
 from StringIO import StringIO
 try:
     #This is in Python 2.6+, but we need it on Python 3
@@ -34,11 +36,70 @@ def add_prefix(key):
     """Dummy key_function for testing index code."""
     return "id_" + key
 
+def gzip_open(filename, format):
+    #At time of writing, under Python 3.2.2 seems gzip.open(filename, mode)
+    #insists on giving byte strings (i.e. binary mode)
+    #See http://bugs.python.org/issue13989
+    if sys.version_info[0] < 3 or format in SeqIO._BinaryFormats:
+        return gzip.open(filename)
+    handle = gzip.open(filename)
+    data = handle.read() #bytes!
+    handle.close()
+    return StringIO(_bytes_to_string(data))
+
+
+if sqlite3:
+    class OldIndexTest(unittest.TestCase):
+        """Testing a pre-built index (make sure cross platform etc).
+
+        >>> from Bio import SeqIO
+        >>> d = SeqIO.index_db("triple_sff.idx", ["E3MFGYR02_no_manifest.sff", "greek.sff", "paired.sff"], "sff")
+        >>> len(d)
+        54
+        """
+        def test_old(self):
+            """Load existing index with no options."""
+            d = SeqIO.index_db("Roche/triple_sff.idx")
+            self.assertEqual(54, len(d))
+
+        def test_old_format(self):
+            """Load existing index with correct format."""
+            d = SeqIO.index_db("Roche/triple_sff.idx", format="sff")
+            self.assertEqual(54, len(d))
+
+        def test_old_format_wrong(self):
+            """Load existing index with wrong format."""
+            self.assertRaises(ValueError, SeqIO.index_db,
+                              "Roche/triple_sff.idx", format="fasta")
+
+        def test_old_files(self):
+            """Load existing index with correct files."""
+            d = SeqIO.index_db("Roche/triple_sff.idx",
+                               ["E3MFGYR02_no_manifest.sff", "greek.sff", "paired.sff"])
+            self.assertEqual(54, len(d))
+
+        def test_old_files_wrong(self):
+            """Load existing index with wrong files."""
+            self.assertRaises(ValueError, SeqIO.index_db,
+                              "Roche/triple_sff.idx", ["a.sff", "b.sff", "c.sff"])
+
+        def test_old_files_wrong2(self):
+            """Load existing index with wrong number of files."""
+            self.assertRaises(ValueError, SeqIO.index_db,
+                              "Roche/triple_sff.idx",
+                              ["E3MFGYR02_no_manifest.sff", "greek.sff"])
+
+
 class IndexDictTests(unittest.TestCase):
     """Cunning unit test where methods are added at run time."""
-    def simple_check(self, filename, format, alphabet):
+    def simple_check(self, filename, format, alphabet, comp):
         """Check indexing (without a key function)."""
-        id_list = [rec.id for rec in SeqIO.parse(filename, format, alphabet)]
+        if comp:
+            h = gzip_open(filename, format)
+            id_list = [rec.id for rec in SeqIO.parse(h, format, alphabet)]
+            h.close()
+        else:
+            id_list = [rec.id for rec in SeqIO.parse(filename, format, alphabet)]
 
         rec_dict = SeqIO.index(filename, format, alphabet)
         self.check_dict_methods(rec_dict, id_list, id_list)
@@ -50,7 +111,8 @@ class IndexDictTests(unittest.TestCase):
 
         #In memory,
         #note here give filenames as list of strings
-        rec_dict = SeqIO.index_db(":memory:", [filename], format, alphabet)
+        rec_dict = SeqIO.index_db(":memory:", [filename], format,
+                                  alphabet)
         self.check_dict_methods(rec_dict, id_list, id_list)
         rec_dict.close()
         del rec_dict
@@ -69,27 +131,37 @@ class IndexDictTests(unittest.TestCase):
         #To disk,
         #note here we give the filename as a single string
         #to confirm that works too (convience feature).
-        rec_dict = SeqIO.index_db(index_tmp, filename, format, alphabet)
+        rec_dict = SeqIO.index_db(index_tmp, filename, format,
+                                  alphabet)
         self.check_dict_methods(rec_dict, id_list, id_list)
         rec_dict.close()
+        rec_dict._con.close() #hack for PyPy
         del rec_dict
 
         #Now reload it...
-        rec_dict = SeqIO.index_db(index_tmp, [filename], format, alphabet)
+        rec_dict = SeqIO.index_db(index_tmp, [filename], format,
+                                  alphabet)
         self.check_dict_methods(rec_dict, id_list, id_list)
         rec_dict.close()
+        rec_dict._con.close() #hack for PyPy
         del rec_dict
 
         #Now reload without passing filenames and format
         rec_dict = SeqIO.index_db(index_tmp, alphabet=alphabet)
         self.check_dict_methods(rec_dict, id_list, id_list)
         rec_dict.close()
+        rec_dict._con.close() #hack for PyPy
         del rec_dict
         os.remove(index_tmp)
     
-    def key_check(self, filename, format, alphabet):
+    def key_check(self, filename, format, alphabet, comp):
         """Check indexing with a key function."""
-        id_list = [rec.id for rec in SeqIO.parse(filename, format, alphabet)]
+        if comp:
+            h = gzip_open(filename, format)
+            id_list = [rec.id for rec in SeqIO.parse(h, format, alphabet)]
+            h.close()
+        else:
+            id_list = [rec.id for rec in SeqIO.parse(filename, format, alphabet)]
 
         key_list = [add_prefix(id) for id in id_list]
         rec_dict = SeqIO.index(filename, format, alphabet, add_prefix)
@@ -122,6 +194,7 @@ class IndexDictTests(unittest.TestCase):
                                   add_prefix)
         self.check_dict_methods(rec_dict, key_list, id_list)
         rec_dict.close()
+        rec_dict._con.close() #hack for PyPy
         del rec_dict
 
         #Now reload it...
@@ -129,6 +202,7 @@ class IndexDictTests(unittest.TestCase):
                                   add_prefix)
         self.check_dict_methods(rec_dict, key_list, id_list)
         rec_dict.close()
+        rec_dict._con.close() #hack for PyPy
         del rec_dict
 
         #Now reload without passing filenames and format
@@ -136,6 +210,7 @@ class IndexDictTests(unittest.TestCase):
                                   key_function=add_prefix)
         self.check_dict_methods(rec_dict, key_list, id_list)
         rec_dict.close()
+        rec_dict._con.close() #hack for PyPy
         del rec_dict
         os.remove(index_tmp)
         #Done
@@ -188,13 +263,22 @@ class IndexDictTests(unittest.TestCase):
         self.assertRaises(NotImplementedError, rec_dict.copy)
         self.assertRaises(NotImplementedError, rec_dict.fromkeys, [])
 
-    def get_raw_check(self, filename, format, alphabet):
-        handle = open(filename, "rb")
-        raw_file = handle.read()
-        handle.close()
+    def get_raw_check(self, filename, format, alphabet, comp):
         #Also checking the key_function here
-        id_list = [rec.id.lower() for rec in \
-                   SeqIO.parse(filename, format, alphabet)]
+        if comp:
+            h = gzip.open(filename, "rb")
+            raw_file = h.read()
+            h.close()
+            h = gzip_open(filename, format)
+            id_list = [rec.id.lower() for rec in \
+                       SeqIO.parse(h, format, alphabet)]
+            h.close()
+        else:
+            h = open(filename, "rb")
+            raw_file = h.read()
+            h.close()
+            id_list = [rec.id.lower() for rec in \
+                       SeqIO.parse(filename, format, alphabet)]
         rec_dict = SeqIO.index(filename, format, alphabet,
                                key_function = lambda x : x.lower())
         self.assertEqual(set(id_list), set(rec_dict.keys()))
@@ -319,33 +403,37 @@ tests = [
     ]
 for filename, format, alphabet in tests:
     assert format in _FormatToRandomAccess
+    tasks = [(filename, None)]
+    if os.path.isfile(filename + ".bgz"):
+        tasks.append((filename + ".bgz","bgzf"))
+    for filename, comp in tasks:
 
-    def funct(fn,fmt,alpha):
-        f = lambda x : x.simple_check(fn, fmt, alpha)
-        f.__doc__ = "Index %s file %s defaults" % (fmt, fn)
-        return f
-    setattr(IndexDictTests, "test_%s_%s_simple" \
-            % (filename.replace("/","_").replace(".","_"), format),
-            funct(filename, format, alphabet))
-    del funct
+        def funct(fn,fmt,alpha,c):
+            f = lambda x : x.simple_check(fn, fmt, alpha, c)
+            f.__doc__ = "Index %s file %s defaults" % (fmt, fn)
+            return f
+        setattr(IndexDictTests, "test_%s_%s_simple" \
+                    % (format, filename.replace("/","_").replace(".","_")),
+                funct(filename, format, alphabet, comp))
+        del funct
 
-    def funct(fn,fmt,alpha):
-        f = lambda x : x.key_check(fn, fmt, alpha)
-        f.__doc__ = "Index %s file %s with key function" % (fmt, fn)
-        return f
-    setattr(IndexDictTests, "test_%s_%s_keyf" \
-            % (filename.replace("/","_").replace(".","_"), format),
-            funct(filename, format, alphabet))
-    del funct
+        def funct(fn,fmt,alpha,c):
+            f = lambda x : x.key_check(fn, fmt, alpha, c)
+            f.__doc__ = "Index %s file %s with key function" % (fmt, fn)
+            return f
+        setattr(IndexDictTests, "test_%s_%s_keyf" \
+                    % (format, filename.replace("/","_").replace(".","_")),
+                funct(filename, format, alphabet, comp))
+        del funct
 
-    def funct(fn,fmt,alpha):
-        f = lambda x : x.get_raw_check(fn, fmt, alpha)
-        f.__doc__ = "Index %s file %s get_raw" % (fmt, fn)
-        return f
-    setattr(IndexDictTests, "test_%s_%s_get_raw" \
-            % (filename.replace("/","_").replace(".","_"), format),
-            funct(filename, format, alphabet))
-    del funct
+        def funct(fn,fmt,alpha,c):
+            f = lambda x : x.get_raw_check(fn, fmt, alpha, c)
+            f.__doc__ = "Index %s file %s get_raw" % (fmt, fn)
+            return f
+        setattr(IndexDictTests, "test_%s_%s_get_raw" \
+                    % (format, filename.replace("/","_").replace(".","_")),
+                funct(filename, format, alphabet, comp))
+        del funct
 
 if __name__ == "__main__":
     runner = unittest.TextTestRunner(verbosity = 2)
